# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gio
import datetime
import enchant
import multiprocessing
from Queue import Empty
from storm.expr import Or
import dexter.backend.models as models

QUEUE = multiprocessing.Queue()  # this queue is used for saving photo files in a separate process


def create_new_contact(first_name, middle_name=None, last_name=None,
                       organization=None, birthday=None, photo_path=None,
                       emails=[], phones=[], addresses=[], imnicks=[], notes=[],
                       store=None):
    """
    create a new contact in the database, all arguments should be unicode
    strings except for birthday with sould be a datetime.date() object, or
    it can also be a unicode string formatted yyyy-mm-dd. emails and phones
    should be a list of tuples containing the value (address/number) and type.
    addresses should be a list of dictionaries containing the address parts and
    the type. Notes should be a list of unicode strings
    """
    global QUEUE
    if enchant.__ver_minor__ < 5:
        dictionary = False
    else:
        dictionary = True
    if store == None:
        store = models.setup_db(models.DATA_DIR)  # create a store if one wasn't provided
    if dictionary:
        try:
            d = enchant.Dict()  # for adding names to user spell check dictionary
            dictionary = True
        except enchant.DictNotFoundError:
            dictionary = False
    new = store.add(models.Contact(first_name))
    if dictionary:
        d.add(first_name)
    if middle_name:
        new.middle_name = middle_name
        if dictionary:
            d.add(middle_name)
    if last_name:
        new.last_name = last_name
        if dictionary:
            d.add(last_name)
    new.create_full_name()
    if organization:
        new.organization = organization
        if dictionary:
            d.add(organization)
    if birthday:
        birthdaylist = birthday.split("-")
        new.birthday = datetime.datetime(int(birthdaylist[0]),
                                         int(birthdaylist[1]),
                                         int(birthdaylist[2]))
    if photo_path:
        store.flush()
        dest_name = "/" + str(new.id) + str(new.first_name).lower() + \
                    str(new.last_name).lower()
        new.photo = unicode(dest_name)
        QUEUE.put([dest_name, photo_path])  # put the photo data into the queue for processing in a separate process
        if not multiprocessing.active_children():
            p = multiprocessing.Process(target=add_photo, args=(QUEUE,))
            p.start()                   # start the photo adding Process if it isn't already running
    for address, type_ in emails:
        if address != "None":
            email = store.add(models.Email(address, type_))
            new.emails.add(email)
    for number, type_ in phones:
        if number != "None":
            phone = store.add(models.Phone(number, type_))
            new.phones.add(phone)
    for adict in addresses:
        if adict["street"] or adict["city"] or adict["state"] or adict["zip"]:
            address = store.add(models.Address(adict["street"], adict["city"],
                                               adict["state"], adict["zip"],
                                               adict["type"]))
            new.addresses.add(address)
    for nick, type_ in imnicks:
        if nick != "None":
            imnick = store.add(models.IMNick(nick, type_))
            new.imnicks.add(imnick)
    for text in notes:
        if text and text.strip() != "None":
            note = store.add(models.Note(text))
            new.notes.add(note)
    store.flush()
    return new.id


def add_photo(q):
    """create a photo for each item in q
    
    This is meant to be run in a separate process
    """
    while True:
        try:
            item = q.get(True, 3)  # wait for 3 seconds to get an item if q is empty
            dest_name = item[0]
            source_path = item[1]
            dest_path = models.IMAGE_DIR + dest_name
            source = gio.File(source_path)
            dest = gio.File(dest_path)
            source.copy(dest, flags=gio.FILE_COPY_OVERWRITE)
        except Empty:
            break           # end the task if the queue is empty for more than 3 seconds


def filter_results(text, results, store):
    """
    take a list of contacts and a text string, and return a filtered
    list of contacts containing that string.

    """
    text = text.lower()
    next_results = []
    for contact in results:
        contact_object = store.find(models.Contact, models.Contact.id == int(contact["id"])).one()
        if unicode(text) in contact["full_name"].lower():
            next_results.append(contact)
        elif contact["organization"] != "None":
            if unicode(text) in contact["organization"].lower():
                next_results.append(contact)
        elif contact_object.emails.count() > 0:
            for email in contact_object.emails:
                if unicode(text) in email.address.lower():
                    next_results.append(contact)
                    break
    return next_results


def search_store(text, store):
    results = []
    search = unicode("%" + text + "%")
    contacts = store.find(models.Contact, Or(models.Contact.full_name.like(search), models.Contact.organization.like(search)))
    emails = store.find(models.Email, models.Email.address.like(search))
    for contact in contacts:
        results.append(contact)
    for email in emails:
        if email.contact not in results:
            results.append(email.contact)
    return results

def get_by_name_email(full_name, email, store):
    results = []
    contacts = store.find(models.Contact, models.Contact.full_name.like(full_name))
    emails = store.find(models.Email, models.Email.address.like(email))
    for contact in contacts:
        results.append(contact)
    for mail in emails:
        if mail.contact not in results:
            results.append(mail.contact)
    return results

def get_birthdays(month, store):
    results = []
    search = unicode("-" + "%02i" % (month) + "-")
    contacts = store.find(models.Contact)
    for contact in contacts:
        if search in str(contact.birthday):
            results.append([contact.full_name, str(contact.birthday)])
    return results

def delete_by_id(store, id_number):
    """delete the given contact and it's children from the given store"""
    contact = store.find(models.Contact, models.Contact.id == id_number).one()
    for email in contact.emails:
        store.remove(email)
    for phone in contact.phones:
        store.remove(phone)
    for address in contact.addresses:
        store.remove(address)
    for imnick in contact.imnicks:
        store.remove(imnick)
    for note in contact.notes:
        store.remove(note)
#    try:
#        os.remove(models.IMAGE_DIR + contact.photo)
#    except:
#        pass
    store.remove(contact)
    store.flush()
